#include "camera.h"
#include "graphics.h"

Camera::Camera(void)
{
	eye = Vector(0,0,5);
	roll = 0.0f;
	track = std::vector<CAMERA_TRACK>();
}

Camera::~Camera(void)
{
	track.clear();
}

void Camera::render(void)
{
	glRotatef(roll, 0,0,1);
	gluLookAt(
			eye.x, eye.y, eye.z,
			center.x, center.y, center.z,
			0, 1, 0
	);
}

void Camera::addTrack(int time, Vector eye, Vector center, float roll)
{
	CAMERA_TRACK *tmpTrack = new CAMERA_TRACK;

	tmpTrack->time = time;
	tmpTrack->eye = eye;
	tmpTrack->center = center;
	tmpTrack->roll = roll;

	track.push_back(*tmpTrack);

	delete tmpTrack;
}

float Camera::frameCalc(float percentage, float startValue, float endValue)
{
	return percentage*(endValue-startValue) + startValue;
}

void Camera::setTrack(int time)
{
	for(unsigned int i = 1; i < track.size(); i++)
	{
		if (time <= track[i].time)
		{
			const float percentage = (time-track[i-1].time)/(float)(track[i].time-track[i-1].time);

			eye.x = frameCalc(percentage, track[i-1].eye.x, track[i].eye.x);
			eye.y = frameCalc(percentage, track[i-1].eye.y, track[i].eye.y);
			eye.z = frameCalc(percentage, track[i-1].eye.z, track[i].eye.z);

			center.x = frameCalc(percentage, track[i-1].center.x, track[i].center.x);
			center.y = frameCalc(percentage, track[i-1].center.y, track[i].center.y);
			center.z = frameCalc(percentage, track[i-1].center.z, track[i].center.z);

			roll = frameCalc(percentage, track[i-1].roll, track[i].roll);

			return;
		}
	}

	eye = track[track.size()-1].eye;
	center = track[track.size()-1].center;
	roll = track[track.size()-1].roll;
}
