#include <math.h>
#include "vector.h"

Vector::Vector(void)
{
	x = y = z = 0.0f;
}

Vector::Vector(const Vector& vec)
{
	x = vec.x;
	y = vec.y;
	z = vec.z;
}

Vector::Vector(float x, float y, float z): x(x), y(y), z(z) {}

float Vector::length(void)
{
	return (float)sqrt(x*x + y*y + z*z);
}

float Vector::distance(const Vector& vec)
{
float dx = x-vec.x;
float dy = x-vec.y;
float dz = x-vec.z;
	//return (float)sqrt((x*x - vec.x*vec.x) + (y*y - vec.y*vec.y) + (z*z - vec.z*vec.z));
return (float)sqrt(dx*dx + dy*dy + dz*dz);
}

Vector Vector::normalize(void)
{
	float tmpl = length();
	if (tmpl==0)
	{
		return Vector(0, 0, 0);
	}

	return Vector(x/tmpl, y/tmpl, z/tmpl);
}

float Vector::dotProduct(const Vector& vec)
{
	return x*vec.x + y*vec.y + z*vec.z;
}

Vector Vector::crossProduct(const Vector& vec)
{
	return Vector(y*vec.z-z*vec.y, -(x*vec.z-z*vec.x), x*vec.y-y*vec.x);
}

Vector Vector::normal(const Vector& vec2, const Vector& vec3)
{
	Vector t1, t2;

	t1.x = x - vec2.x;
	t1.y = y - vec2.y;
	t1.z = z - vec2.z;
//	t1 = t1.normalize();

	t2.x = vec2.x - vec3.x;
	t2.y = vec2.y - vec3.y;
	t2.z = vec2.z - vec3.z;
//	t2 = t2.normalize();

	t1 = t1.crossProduct(t2);

	return t1.normalize();
}

float Vector::angle(Vector& vec)
{
	return acos(dotProduct(vec) / (length() * vec.length()));
}

bool Vector::operator == (const Vector& vec)
{
	return ((x==vec.x) && (y==vec.y) && (z==vec.z))?true:false;
}

void Vector::operator = (const Vector& vec)
{
	x = vec.x;
	y = vec.y;
	z = vec.z;
}

Vector Vector::operator + (const Vector& vec)
{
	return Vector(x+vec.x, y+vec.y, z+vec.z);
}

Vector Vector::operator += (const Vector& vec)
{
	x += vec.x;
	y += vec.y;
	z += vec.z;
	return Vector(x, y, z);
}

Vector Vector::operator - (const Vector& vec)
{
	return Vector(x-vec.x, y-vec.y, z-vec.z);
}

Vector Vector::operator -= (const Vector& vec)
{
	x -= vec.x;
	y -= vec.y;
	z -= vec.z;
	return Vector(x, y, z);
}

Vector Vector::operator * (const float& a)
{
	return Vector(x*a, y*a, z*a);
}

Vector Vector::operator *= (const float& a)
{
	x *= a;
	y *= a;
	z *= a;
	return Vector(x, y, z);
}

Vector Vector::operator / (const float& a)
{
	return Vector(x/a, y/a, z/a);
}

Vector Vector::operator /= (const float& a)
{
	x /= a;
	y /= a;
	z /= a;
	return Vector(x, y, z);
}
